#ifndef ds50daq_DAQ_V172xDriver_hh
#define ds50daq_DAQ_V172xDriver_hh

#include "fhiclcpp/fwd.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "DS50FragmentGenerator.hh"
#include "ds50daq/DAQ/V172xFragment.hh"
#include "ds50daq/DAQ/StatisticsHelper.hh"

#include <string>
#include <vector>

namespace ds50 {
  // V172xDriver reads DS50 events from the board
  class V172xDriver : public DS50FragmentGenerator {
    public:
      static const std::string READ_TIME_STAT_KEY;
      static const std::string FRAGMENT_CREATE_TIME_STAT_KEY;
      static const std::string MEMCPY_TIME_STAT_KEY;

      explicit V172xDriver(const fhicl::ParameterSet &);
      virtual ~V172xDriver() noexcept;

    private:
      void start_ () override; 
      void stop_ () override; 
      bool getNext__ (artdaq::FragmentPtrs & output) override;

      int link_type_;
      bool vme_;
      int handle_;
      int sample_rate_MHz_;
      int bit_width_;
      bool sw_acquisition_;
      bool sw_trigger_;
      int irqwait_ms_, irqwait_checks_;
      char *event_buffer_;
      uint32_t buffer_size_;
      bool generate_fragment_;
      uint8_t ev_offset_;
      bool pattern_check_;
      V172xFragment::metadata metadata_;
      artdaq::detail::RawFragmentHeader::type_t fragment_type_;


      void init_link (fhicl::ParameterSet const &);
      void init_metadata (fhicl::ParameterSet const &);
      void init_channels (fhicl::ParameterSet const &);
      void init_trigger (fhicl::ParameterSet const &);
      void init_buffers (fhicl::ParameterSet const &);
      void close_link ();

      uint32_t get_register (uint16_t reg); 
      void set_register (uint16_t reg, uint32_t value);
      uint32_t set_register_bits (uint16_t reg, uint32_t bits);
      bool wait_irq ();

      // attributes and methods for statistics gathering & reporting
      StatisticsHelper statsHelper_;
      std::string buildStatisticsString_();
  };
}
#endif /* ds50daq_DAQ_V172xDriver_hh */
