#ifndef CAEN_IO_H_
#define CAEN_IO_H_
#include <string>
#include <stdint.h>
#ifndef LINUX
#define LINUX
#include "CAENVMEtypes.h"
#undef LINUX
#else
#include "CAENVMEtypes.h"
#endif

namespace detail {
  class caen_io {
    public:
      caen_io () = delete;
      caen_io (const caen_io&) = delete;
      caen_io& operator=(const caen_io&) = delete;
      caen_io (CVAddressModifier am, CVDataWidth dm, uint32_t vme_base, const std::string& link_type, int link_number, int link_node);
      ~caen_io ();

      uint32_t vme_base () const { return vme_base_; }
      void set_vme_base (int vme_base) { vme_base_ = vme_base; }

      void write_register (uint32_t offset, uint32_t value, CVDataWidth dm);
      uint32_t set_register_bits (uint32_t offset, uint32_t mask, CVDataWidth dm);
      uint32_t clear_register_bits (uint32_t offset, uint32_t mask, CVDataWidth dm);
      uint32_t read_register (uint32_t offset, CVDataWidth dm);

      void write_register (uint32_t offset, uint32_t value) { write_register (offset, value, dm_); }
      uint32_t set_register_bits (uint32_t offset, uint32_t mask) { return set_register_bits (offset, mask, dm_); }
      uint32_t clear_register_bits (uint32_t offset, uint32_t mask) { return clear_register_bits (offset, mask, dm_); }
      uint32_t read_register (uint32_t offset) { return read_register (offset, dm_); }

      enum block_mode {
	sequential,
	sequential_fifo,
	blt,
	blt_fifo
      };
      ssize_t read_block (uint32_t offset, void* destination, size_t bytes, block_mode);
      ssize_t read_block (uint32_t offset, void* destination, size_t bytes, block_mode, CVAddressModifier, CVDataWidth);

      void enable_outputs ();
      void set_outputs ();
      void clear_outputs ();

      bool wait_irq (CVIRQLevels irq, uint32_t timeout_ms);
      bool wait_irq (CVIRQLevels irq, uint32_t timeout_ms, uint32_t& vector, CVDataWidth dm);
      void vme_sysreset ();

#ifdef _IO_DEBUG_
      void debug (bool on) { debug_ = on; }
#endif
    private:
      CVAddressModifier am_;
      CVDataWidth dm_;
      int handle_;
      uint32_t vme_base_;

      uint32_t mask_;
      size_t b4b_;
#ifdef _IO_DEBUG_
      bool debug_;
#endif
  };
}

#endif
