#!/usr/bin/env ruby

require 'optparse'

# Read the input and return a Hash with 3 arrays of integers.
def read_input(filename)
  sym, code, bits = [], [], []
  File.open(filename, "r") do |f|
    f.each_line do |line|
      s, c, b = line.split
      sym << Integer(s)
      code << Integer(c)
      bits << Integer(b)
    end
  end
  { :sym => sym,
    :code => code,
    :bits => bits }
end

# Write a FHiCL file (to standard out) containing the contents of the
# input Hash. All keys are expected to be Symbols, and values are
# Arrays.
def write_output(h, name)
  puts "BEGIN_PROLOG"
  print name, ":\n{\n"
  h.each_pair do |k, v|
    print "  ", k, ": [", v.join(", "), "]\n"
  end
  puts "}"
  puts "END_PROLOG"
end

if __FILE__ == $0

  options = {}
  optparse = OptionParser.new do |opts|
    opts.banner = "Usage: table_to_fcl.rb [options] input-file param-name"

    opts.on("-v", "--[no-]verbose", "Run verbosely") do |v|
      options[:verbose] = v
    end
  end

  opts = optparse.parse!

  if ARGV.size < 2
    puts optparse.banner
  end

  input = read_input(ARGV[0])
  write_output(input, ARGV[1])
end
