#include "ds50daq/DAQ/AggregatorApp.hh"
#include "ds50daq/DAQ/Aggregator.hh"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "artdaq/DAQrate/RHandles.hh"
#include "ds50daq/DAQ/Config.hh"

#include "art/Framework/Art/artapp.h"

#include <iostream>

ds50::AggregatorApp::AggregatorApp(int mpi_rank, MPI_Comm local_group_comm) :
  mpi_rank_(mpi_rank), local_group_comm_(local_group_comm) { }

// *******************************************************************
// *** The following methods implement the state machine operations.
// *******************************************************************

bool ds50::AggregatorApp::do_initialize(fhicl::ParameterSet const& pset)
{
  report_string_ = "";

  //aggregator_ptr_.reset(nullptr);
  if (aggregator_ptr_.get() == 0) {
    aggregator_ptr_.reset(new Aggregator(mpi_rank_, local_group_comm_));
  }
  external_request_status_ = aggregator_ptr_->initialize(pset);
  if (!external_request_status_) {
    report_string_ = "Error initializing the Aggregator with ";
    report_string_.append("ParameterSet = \"" + pset.to_string() + "\".");
  }

  return external_request_status_;
}

bool ds50::AggregatorApp::do_start(art::RunID id)
{
  report_string_ = "";
  external_request_status_ = aggregator_ptr_->start(id);
  if (!external_request_status_) {
    report_string_ = "Error starting the Aggregator for run ";
    report_string_.append("number ");
    report_string_.append(boost::lexical_cast<std::string>(id.run()));
    report_string_.append(".");
  }

  aggregator_future_ =
    std::async(std::launch::async, &Aggregator::process_fragments,
               aggregator_ptr_.get());

  return external_request_status_;

}

bool ds50::AggregatorApp::do_stop()
{
  report_string_ = "";
  external_request_status_ = aggregator_ptr_->stop();
  if (!external_request_status_) {
    report_string_ = "Error stopping the Aggregator";
  }

  aggregator_future_.get();
  return external_request_status_;
}

bool ds50::AggregatorApp::do_pause()
{
  report_string_ = "";
  external_request_status_ = aggregator_ptr_->pause();
  if (!external_request_status_) {
    report_string_ = "Error pausing the Aggregator";
  }

  aggregator_future_.get();
  return external_request_status_;
}

bool ds50::AggregatorApp::do_resume()
{
  report_string_ = "";
  external_request_status_ = aggregator_ptr_->resume();
  if (!external_request_status_) {
    report_string_ = "Error resuming the Aggregator";
  }

  aggregator_future_ =
    std::async(std::launch::async, &Aggregator::process_fragments,
               aggregator_ptr_.get());

  return external_request_status_;
}

bool ds50::AggregatorApp::do_shutdown()
{
  report_string_ = "";
  external_request_status_ = aggregator_ptr_->shutdown();
  if (!external_request_status_) {
    report_string_ = "Error shutting down the Aggregator";
  }

  return external_request_status_;
}

bool ds50::AggregatorApp::do_soft_initialize(fhicl::ParameterSet const&)
{
  return true;
}

bool ds50::AggregatorApp::do_reinitialize(fhicl::ParameterSet const&)
{
  return true;
}

std::string ds50::AggregatorApp::report(std::string const& which) const
{
  // if there is an outstanding error, return that
  if (report_string_.length() > 0) {
    return report_string_;
  }

  if (which == "event_count" || which == "run_duration" || which == "file_size") {
    if (status() == "Running") {
      return aggregator_ptr_->report(which);
    }
    else {
      return "-1";
    }
  }

  return "Unknown request: " + which;
}
