#ifndef ds50daq_DAQ_V1495Driver_hh
#define ds50daq_DAQ_V1495Driver_hh

#include <memory>
#include "fhiclcpp/fwd.h"
#include "artdaq/DAQdata/Fragments.hh"
#include "DS50FragmentGenerator.hh"
#include "ds50daq/DAQ/V1495Fragment.hh"

namespace detail {
  class caen_io;
}

namespace ds50 {
  // V1495Driver reads DS50 events from the board
  class V1495Driver : public DS50FragmentGenerator {
    public:
      explicit V1495Driver(fhicl::ParameterSet const &);
      virtual ~V1495Driver() noexcept;

    private:
      void start_ () override;
      void stop_ () override;
      void pause_ () override;
      void resume_ () override;
      void shutdown_ () override;
      bool getNext__(artdaq::FragmentPtrs & output) override;

      std::size_t current_event_num_;

      std::unique_ptr<detail::caen_io> caen_io_;
      bool dont_stop_;
      int irqwait_ms_, irqwait_checks_;
      int irqlost_;
      V1495Fragment::metadata metadata_;


      void init_link (fhicl::ParameterSet const &);
      void init_metadata (fhicl::ParameterSet const &);
      void init_trigger (fhicl::ParameterSet const &);
      void init_channels (fhicl::ParameterSet const &);
      void close_link ();
  };
}
#endif /* ds50daq_DAQ_V1495Driver_hh */
