#!/usr/bin/env ruby

require "open3"

def fileInUse(filePath)
  # Determine if a file is in use using the lsof utility.  If the file isn't in
  # use the utility will return no output to stdout.
  lsofResult = nil
  lsofError = nil
  Open3.popen3("/usr/sbin/lsof %s" % [filePath]) { |stdin, stdout, stderr|
    lsofResult = stdout.read
    lsofError = stderr.read
  }
  if lsofError != ""
    puts "lsof ERROR: " + lsofError
    return true
  end
  if lsofResult != ""
    return true
  end

  # 15-May-2013, KAB - adding double-check (since I saw an instance where a
  # current file was accidentally deleted).  In this test, we compare the
  # current time to the last modified time of the file.
  lastModifyTime = nil
  Open3.popen3("stat --format='%%Y' %s" % [filePath]) { |stdin, stdout, stderr|
    lastModifyTime = stdout.read
  }
  #puts lastModifyTime
  currentTime = nil
  Open3.popen3("date '+%s'") { |stdin, stdout, stderr|
    currentTime = stdout.read
  }
  #puts currentTime
  timeDiff = Integer(currentTime) - Integer(lastModifyTime)
  #puts timeDiff
  if timeDiff < 30
    return true
  end

  return false
end

if __FILE__ == $0
  if ARGV.length != 1
    puts "Usage:"
    puts "  %s /path/to/data/directory" % [__FILE__]
    exit
  end

  fileEntries = Dir.entries(ARGV[0])
  fileEntries.each do |fileEntry|
    # Assume that the DAQ writes out files starting with "ds50daq"
    # and ending with "root"
    if fileEntry =~ /^ds50daq.*root$/ || fileEntry =~ /^RootOutput.*root$/
      fullPath = File.join(ARGV[0], fileEntry)
      if ! fileInUse(fullPath)
        puts "%s" % [fileEntry]
      end
    end
  end
end
