#include <iostream>
#include <string>

#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "xmlrpc_commander.hh"


int main (int argc, char *argv[]) try {
  fhicl::ParameterSet pset = mf::MessageFacilityService::ConfigurationFile ("logging.fcl", mf::MessageFacilityService::logCF ());
  mf::StartMessageFacility (mf::MessageFacilityService::MultiThread, pset);

  std::string usage = std::string(argv[0]) + " -p port_number <other-options>";
  boost::program_options::options_description desc(usage);

  desc.add_options ()
    ("port,p", boost::program_options::value<unsigned short>(), "Port number")
    ("help,h", "produce help message");
  
  boost::program_options::variables_map vm;
  try {
    boost::program_options::store (boost::program_options::command_line_parser(argc, argv).options(desc).run(), vm);
    boost::program_options::notify (vm);
  } catch (boost::program_options::error const& e) {
    mf::LogError ("Option") << "exception from command line processing in " << argv[0] << ": " << e.what() << std::endl;
    return 1;
  }

  if (vm.count("help")) {
    std::cout << desc << std::endl;
    return 0;
  }

  if (!vm.count("port")) {
    mf::LogError ("Option") << argv[0] << " port number not suplied" << std::endl << "For usage and an options list, please do '" << argv[0] <<  " --help'" << std::endl;
    return 1;
  }

  mf::SetApplicationName ("xmlrpc_main-" + boost::lexical_cast<std::string>(vm["port"].as<unsigned short> ()));
  
  xmlrpc_commander(vm["port"].as<unsigned short> ()) ();
  
  return 0;
} catch (std::exception const& x) {
  mf::LogError ("XMLRPC") << "Exception (type std::exception) caught in xmlrpc-main:" << x.what () << std::endl;
  return 1;
} catch (std::string& x) {
  mf::LogError ("XMLRPC") << "Exception (type string) caught in xmlrpc-main:" << x << std::endl;
  return 1;
} catch (char const *m) {
  mf::LogError ("XMLRPC") << "Exception (type char const*) caught in driver: " << (m ? m : "nil") << std::endl;
  return 1;
} catch (...) {
  mf::LogError ("XMLRPC") << "Unknown exception caught in driver: " << std::endl;
  return 1;
}

