#ifndef _DS50DB_INTERFACE
#define _DS50DB_INTERFACE
#include <string>
#include <memory>
#include <pqxx/pqxx>
#include <mutex>

#include "ds50daq/online/ArtServices/ds50db_result.hh"

namespace ds50 {
  namespace db {
    class interface {
      public:
	interface (const std::string& host, const std::string& db, const std::string& user, const std::string& pwd);

	int current_run () const;

	int latest_run (const std::string& table_, const std::string& version = "");

	result latest (const std::string& table, const std::string& version = "");

	result current (const std::string& table, const std::string& version = "");

	result insert (const result& r, const std::string& version);

	template<typename T> std::string quote (const T& t) const;

	result query (const std::string& q, bool autocommit = false);

	typedef pqxx::lazyconnection connection;
	class work {
	  public:
	    work (connection& c, bool autocommit);
	    ~work ();
	    result exec (const std::string& q);
	    void commit ();
	  private:
	    pqxx::work w_;
	    std::lock_guard<std::mutex> lk_;
	    bool autocommit_;
	    static std::mutex mutex_;
	};

	std::unique_ptr<work> transaction (bool autocommit);

      protected:
	void set_current_run (int run);

      private:
	std::unique_ptr<connection> c_;
	int current_run_;
    };
    std::mutex interface::work::mutex_;
  }
}

inline int ds50::db::interface::current_run () const { return current_run_; }

template<typename T> std::string ds50::db::interface::quote (const T& t) const { 
  try {
    return c_->quote (t); 
  } catch (const std::runtime_error& e) {
    throw cet::exception("dbi::quote") << e.what ();
  }
}

void ds50::db::interface::set_current_run (int run) { current_run_ = run; }
#endif
